/* Created on 27-set-2005 */
package jtransfer;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.zip.Adler32;

import javax.net.ssl.SSLSocket;

public class ReceiveFile implements Service,Runnable {

	public static int LENGTH_BUFFER = 2048;
	
	private String pathToSaveTo;
	private int portToConnect;
	private String ipToConnect;
	private String info;
	private File file;
	private boolean running;
	private boolean validating;
	private long lengthOfFile;
	private Set osservatori;
	private long transfered;
	private Socket slave;
	private long waitedCrc;
	private long realCrc;
	
	public ReceiveFile() {
		this.lengthOfFile = 0;
		this.osservatori = new HashSet();
		this.transfered = 0;
		
	}
	
	public String getIpToConnect() {
		return ipToConnect;
	}
	public void setIpToConnect(String ipToConnect) {
		this.ipToConnect = ipToConnect;
	}
	public String getPathToSaveTo() {
		return pathToSaveTo;
	}
	public void setPathToSaveTo(String pathToSaveTo) {
		this.pathToSaveTo = pathToSaveTo;
	}
	public int getPortToConnect() {
		return portToConnect;
	}
	public void setPortToConnect(int portToConnect) {
		this.portToConnect = portToConnect;
	}

	public boolean perform() {
		boolean variabileDiRitorno = true;
		
		try {
			this.waitedCrc = 0;
			this.realCrc = 0;
			this.running = true;
			this.transfered = 0; 
			this.lengthOfFile = 0; 
			info = "Running..";
			notifyObservers();
			slave = new Socket(this.ipToConnect,this.portToConnect); // new SSLSocket(this.ipToConnect,this.portToConnect);
			System.out.println("Dati pronti");
			String receiveCommand;
			String sendCommand;
			info = "Connecting to " + this.ipToConnect + " on port " + this.portToConnect + " done";
			notifyObservers();
			//	Connessione effettuata
			BufferedReader inslave = new BufferedReader(new InputStreamReader(slave.getInputStream()));
			PrintStream outslave = new PrintStream (slave.getOutputStream());
			System.out.println("Stream pronti");
			// Uso di password?
			sendCommand="JTRANSFER";
			outslave.println(sendCommand);
			if (this.running == false)
				throw new IOException();
			
			receiveCommand = inslave.readLine();
			
			
			if (receiveCommand.equals("ACK")) {
				this.validating = true;
				System.out.println("Riscontro ok");
			} else {
				return false;
			}
			
			if (this.validating) {
				info = "Validation ok.. Gathering information";
				notifyObservers();
			// Lettura info file
			readFileInformation(inslave,outslave);
			// Trasferisci file
			
			inslave = null;
			outslave = null;
			
			byte[] b = new byte[2*ReceiveFile.LENGTH_BUFFER];
			FileOutputStream fileWrite = new FileOutputStream(this.file);
			InputStream insock = slave.getInputStream();
			System.out.println("Buffer disponibile:" + insock.available());
			int lunghezza = insock.read(b);
			System.out.println("Quanti byte letti:" + lunghezza);
			info = "Transfering..";
			notifyObservers();
				while (lunghezza > 0 && this.running) {
					fileWrite.write(b,0,lunghezza);
					this.transfered += lunghezza;
					System.out.println(this.transfered);
					notifyObservers();
					lunghezza = insock.read(b);
				}
			
			} else {
				return false;
			}
			System.out.println("End ok!");
			info = "Transfering.. Done";
			
		} catch (UnknownHostException e) {
			variabileDiRitorno=false;
			e.printStackTrace(System.out);
			this.info = "Errore di collegamento";
			
		} catch (IOException e) {
			variabileDiRitorno=false;
			e.printStackTrace(System.out);
			this.info = "Errore di collegamento";
			
		}
		finally { 
			this.running = false; 
			this.validating = false; 
			
			try {
				checkCrc();
				if (slave != null)
					slave.close();
				this.slave = null;
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				System.out.println("ecce");
			}
			notifyObservers();
			return variabileDiRitorno;
		}
	}

	private void checkCrc() throws IOException {
		Adler32 adl = new Adler32();
		adl.reset();
		byte[] b = new byte[8192];
		if (this.file != null) {
			FileInputStream fileRead = new FileInputStream(this.file);
			int lunghezza = fileRead.read(b);
			while (lunghezza != -1) {
				adl.update(b,0,lunghezza);
				lunghezza = fileRead.read(b);
			}
			this.realCrc = adl.getValue();
			if (this.realCrc != this.waitedCrc)
				info = "Error! Real CRC is " + this.realCrc;
			else
				info = "CRC control passed. All done!";
		} else {
			// TODO
		}
	}

	public void addObserver(Osservatore o) {
		this.osservatori.add(o);
	}
	
	private void readFileInformation(BufferedReader inslave, PrintStream outslave) throws IOException {
			String filename = this.pathToSaveTo + "/" + inslave.readLine();
			File f = new File(filename);
			int i = 1;
			while (f.exists()) {
				f = new File(filename + "-" + i);
				i++;
			}
			this.file = f;
			this.file.createNewFile();
			this.lengthOfFile = Long.parseLong(inslave.readLine());
			this.waitedCrc = Long.parseLong(inslave.readLine());
			System.out.println("Length:" + this.lengthOfFile);
			System.out.println(f.getAbsolutePath());
			// inviare un riscontro?
			outslave.println("ACK");
	}

	private void notifyObservers() {
		for (Iterator it = this.osservatori.iterator();it.hasNext();) {
			Osservatore o = (Osservatore) it.next();
			o.statusCheck(this);
		}
		
	}
	
	public boolean isRunning() {
		return this.running;
	}

	public void setRunning(boolean b) {
		this.running = b;
	}
	
	public boolean isValidating() {
		return this.validating;
	}

	public long transfered() {
		return this.transfered;
	}

	public long lengthOfFile() {
		return this.lengthOfFile;
	}
	
	public String getNomeFile() {
		if (this.file != null) 
			return this.file.getName(); 
		else return "";
	}

	public void closeAll() {
		
	}

	public String getInfo() {
		return this.info;
	}

	public void stopRunning() {
		this.setRunning(false);
	}
	public void run() {
		this.perform();
	}
	
	public long getCrc() {
		return this.waitedCrc;
	}
}
