/* Created on 27-set-2005 */
package jtransfer;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.zip.Adler32;

public class SendFile implements Service,Runnable {

	public static int LENGTH_BUFFER = 2048;
	private String info;
	private int portToListenTo;
	private File file;
	private boolean running;
	private boolean validating;
	private long lengthOfFile;
	private Set osservatori;
	private long transfered;
	private Socket slave;
	private long crc;
	
	public SendFile() {
		this.lengthOfFile =0;
		this.osservatori = new HashSet();
		this.transfered = 0;
		this.crc = 0;
		this.slave = null;
	}

	public void setRunning(boolean b) {
		this.running = b;
	}
	
	public File getFile() {
		return file;
	}
	public void setFile(String filenamePath) {
		this.file = new File(filenamePath);
	}
	public int getPortToListenTo() {
		return portToListenTo;
	}
	public void setPortToListenTo(int portToListenTo) {
		this.portToListenTo = portToListenTo;
	}
	public boolean isRunning() {
		return running;
	}
	
	public long getLengthOfFile() {
		return lengthOfFile;
	}
	public boolean isValidating() {
		return validating;
	}

	
	public long lengthOfFile() {
		return this.lengthOfFile;
	}

	public boolean perform()  {
		boolean variabileDiRitorno = true;
		ServerSocket server = null;
		try {
			this.transfered = 0; 
			this.lengthOfFile = 0; 
			this.running = true;
			info = "Running..";
			notifyObservers();
			String receiveCommand;
			String sendCommand;
			this.lengthOfFile = this.file.length();
			this.crc = calculateCrc();
			System.out.println("Dati inizializzati");

			
			server = new ServerSocket(this.portToListenTo);
			System.out.println("Socket Pronto");
			if (this.running) {
				System.out.println("In attesa di connessioni");
				slave = server.accept();
				
			} else 
				throw new IOException();
			
			System.out.println("Connessione effettuata");
			info = "Receiving connection from " + slave.getInetAddress();
			notifyObservers();
			// Connessione effettuata
			BufferedReader inslave = new BufferedReader(new InputStreamReader(slave.getInputStream()));
			PrintStream outslave = new PrintStream (slave.getOutputStream());
			// Preparazione trasferimento, stream pronti
			System.out.println("Stream pronti");
			// Uso di password?
			receiveCommand = inslave.readLine();
			if (receiveCommand.equals("JTRANSFER")) {
				
				sendCommand = "ACK";
				outslave.println(sendCommand);
				sendFileInformation(outslave);
				if (inslave.readLine().equals("ACK")) {
					this.validating = true;
					System.out.println("Riscontro effettuato");
				}
			}
			
			if (this.validating) {
				inslave = null;
				outslave = null;
				info = "Validating connection from " + slave.getInetAddress();
				notifyObservers();
				byte[] b = new byte[2*SendFile.LENGTH_BUFFER];
				OutputStream send = slave.getOutputStream();
				FileInputStream fileRead = new FileInputStream(this.file);
				// leggi il file e invialo
				int lunghezza = fileRead.read(b);
				info = "Transfering..";
				notifyObservers();
				while (lunghezza != -1) {
					send.write(b,0,lunghezza);
					
					this.transfered += lunghezza;
					System.out.println(this.transfered);
					notifyObservers();
					lunghezza = fileRead.read(b);
				}
			} else {
				return false;
			}
			System.out.println("End ok");
			info = "Transfering..done";
			
		} catch (IOException e) {
			if (slave != null) slave.close();
			System.out.println("Eccezione!");
			e.printStackTrace(System.out);
			this.info = "Errore di collegamento";
			variabileDiRitorno=false;
		} 
		finally { 
			this.running = false; 
			this.validating = false; 
			try {
				if (slave != null)
					slave.close();
				if (server != null)
					server.close();
				this.slave = null;
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			notifyObservers();
			return variabileDiRitorno;
		}
	}

	private long calculateCrc() throws IOException {
		if (this.file != null) {
		Adler32 adl = new Adler32();
		adl.reset();
		byte[] b = new byte[8192];
		FileInputStream fileRead = new FileInputStream(this.file);
		int lunghezza = fileRead.read(b);
		while (lunghezza != -1) {
			adl.update(b,0,lunghezza);
			lunghezza = fileRead.read(b);
		}
		return adl.getValue();
		} else throw new IOException();
	}

	public void stopRunning() {
		this.setRunning(false);
		if (this.slave != null)
			try {
				slave.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
	}
	
	private void notifyObservers() {
		for (Iterator it = this.osservatori.iterator();it.hasNext();) {
			Osservatore o = (Osservatore) it.next();
			o.statusCheck(this);
		}
		
	}

	public long transfered() {
		return this.transfered;
	}
	
	private void sendFileInformation(PrintStream outslave) {
		outslave.println(this.file.getName()); // 1
		outslave.println(this.lengthOfFile); // 2
		outslave.println(this.crc);
	}

	public String getNomeFile() {
		if (this.file != null) 
			return this.file.getName(); 
		else return "";
	}

	public void addObserver(Osservatore o) {
		this.osservatori.add(o);
	}

	public void closeAll() {
		// TODO Auto-generated method stub
	}

	public String getInfo() {
		return this.info;
	}

	public void run() {
		this.perform();
	}

	public long getCrc() {
		return crc;
	}
	
}
